﻿--[[
          Toxynas 'I interrupt that'
          Version 0.0.11 (based on Elphies Version 3.3.9)
--]]

-------------------------------------------------------------------------------
-- A few "local global" variables for this lua-file:                         --
-------------------------------------------------------------------------------
local TIIT_color   = '|cffafdfd0';   -- addon-color for the mesages
local TIIT_name    = "Toxyna's 'I interrupted that'";
                                     -- addon titel used for messages (from .toc)
local TIIT_version = '0.0.11'        -- version of the addon
local TIIT_date    = '2012/09/27'    -- version of the addon
local L            = select(2, ...); -- For internationalization

-------------------------------------------------------------------------------
-- Prints in TIIT's color:                                                   --
-------------------------------------------------------------------------------
local function c_print(message)
    print(TIIT_color .. message .. "|r");
end

-------------------------------------------------------------------------------
-- The Database for this addon - persistence                                 --
-------------------------------------------------------------------------------
ToxynasIInterruptedThatDB = {
    isActive                    = 1,
                                  -- addon is active
    interruptOutput             = 'Emote',
                                  -- addon's output channel is emote
    message                     = L['PERSON interrupts the spell SPELL from MOB!'],
                                  -- message for other channels than emote
    announceInterruptsFromAll   = 0,
                                  -- announce interrupts from all party members
}

-------------------------------------------------------------------------------
-- Prints the greeting, when the user logs in or reloads the gui.            --
-------------------------------------------------------------------------------
c_print(TIIT_name .. " -- " .. L["/tiit for more."]);

-------------------------------------------------------------------------------
-- Creates the frame (I don't know what for...)                              --
-------------------------------------------------------------------------------
local ToxynasIInterruptedThat = CreateFrame("Frame");

-------------------------------------------------------------------------------
--                                                                           --
-------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
    local dispatch = self[event];

    if dispatch then
        dispatch(self, ...);
    end
end

-------------------------------------------------------------------------------
--                                                                           --
-------------------------------------------------------------------------------
ToxynasIInterruptedThat:SetScript("OnEvent", OnEvent);
ToxynasIInterruptedThat:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
ToxynasIInterruptedThat:RegisterEvent("ADDON_LOADED");

-------------------------------------------------------------------------------
-- The function ToxynasIInterruptedThat:ADDON_LOADED is called, when the     --
-- addon is loaded and it provides default values, if the stored DB-Values   --
-- are not found.                                                            --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat:ADDON_LOADED(...)
    if not ToxynasIInterruptedThatDB.isActive then
        ToxynasIInterruptedThatDB.isActive = 1;
    end
    if not ToxynasIInterruptedThatDB.interruptOutput then
        ToxynasIInterruptedThatDB.interruptOutput = 'Emote';
    end
    if not ToxynasIInterruptedThatDB.message then
        ToxynasIInterruptedThatDB.message = L['PERSON interrupts the spell SPELL from MOB!'];
    end
    if not ToxynasIInterruptedThatDB.announceInterruptsFromAll then
        ToxynasIInterruptedThatDB.announceInterruptsFromAll = 0;
    end
    self:UnregisterEvent("ADDON_LOADED");
end

-------------------------------------------------------------------------------
-- Reacts ar unfiltered combat log events.                                   --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat:COMBAT_LOG_EVENT_UNFILTERED(...)
    ---------------------------------------------------------------------------
    -- if the addon is not active : return                                   --
    ---------------------------------------------------------------------------
    if ToxynasIInterruptedThatDB.isActive == 0 then
        return;
    end

    ---------------------------------------------------------------------------
    -- get some of the event parameters:                                     --
    ---------------------------------------------------------------------------
    local eventType        = select(2, ...);
    local interruptingUser = select(5, ...);
    local interruptedUser  = select(9, ...);
    local spellID          = select(15, ...);
    local interruptedSpell = GetSpellLink(spellID);

    ---------------------------------------------------------------------------
    -- handel the event SPELL_INTERRUPT:                                     --
    ---------------------------------------------------------------------------
    if eventType == "SPELL_INTERRUPT" then
        -----------------------------------------------------------------------
        -- if someone else interrupts and we don't want to tell: return      --
        -----------------------------------------------------------------------
        if ToxynasIInterruptedThatDB.announceInterruptsFromAll == 0 and
            interruptingUser ~= UnitName("player") then
                return;
        end

        -----------------------------------------------------------------------
        -- if the interrupter is not in our party/raid: return               --
        -----------------------------------------------------------------------
        if UnitInRaid("player")  and not UnitInRaid(interruptingUser)  or
           UnitInParty("player") and not UnitInParty(interruptingUser) then
            return;
        end

        -----------------------------------------------------------------------
        -- handle the 'Emote' mode:
        -----------------------------------------------------------------------
        if ToxynasIInterruptedThatDB.interruptOutput == 'Emote' then
            -------------------------------------------------------------------
            -- the following placeholder can be used in the Emote-Message:   --
            -- #INTERRUPTED_USER#  the user or mob, that was casting and was --
            --                     interrupted by the player                 --
            -- #INTERRUPTED_SPELL# the spell that user or mob was casting    --
            -- #INTERRUPTING_USER# the player's name                         --
            -------------------------------------------------------------------
            local message; -- the message that will be shown as a emote

            -- the player has interrupted:
            if interruptingUser == UnitName("player") then
                local localizedClass, englishClass = UnitClass("player");
                if "SHAMAN" == englishClass then
                    message = L['sends a burst of wind to #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "ROGUE" == englishClass then
                    -- Rogues can also interrupt via Gauge, Solar Plexus,
                    -- Cheap Shot, or Garotte, but only Kick is a
                    -- "SPELL_INTERRUPT" and I yet have no idea, how to get
                    -- the other four abilities.
                    message = L['kicks #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "MAGE" == englishClass then
                    message = L['speaks a counterspell at #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "WARRIOR" == englishClass then
                    -- Warriors can interrupt spells via "Zuschlagen"
                    -- (Spell id 6552) and "Unterbrechender Ruf" (Spell id 102060),
                    -- so I provide two different emotes for them:
                    local interruptSpellId = select(12, ...);
                    if interruptSpellId == 6552 then -- Zuschlagen
                        message = L['pummels #INTERRUPTED_USER# '
                                .. 'and interrupts #INTERRUPTED_SPELL#.'];
                    elseif interruptSpellId == 102060 then -- Unterbrechender Ruf
                        message = L['shouts loud at #INTERRUPTED_USER# '
                                .. 'and interrupts #INTERRUPTED_SPELL#.'];
                    else
                        message = L['interrupts #INTERRUPTED_USER# '
                                .. 'on casting #INTERRUPTED_SPELL#.'];
                    end
                else
                    message = L['interrupts #INTERRUPTED_USER# '
                            .. 'on casting #INTERRUPTED_SPELL#.'];
               end

            -- someone else has interrupted:
            else
                message = L['admired, how #INTERRUPTED_USER# '
                        .. 'is interrupting #INTERRUPTED_USER# on casting '
                        .. '#INTERRUPTED_SPELL#.'];
            end

            -- replace the placeholders:
            if nil ~= string.find(message, '#INTERRUPTED_USER#', 1, true) then
                message = string.gsub(message, '#INTERRUPTED_USER#', interruptedUser);
            end
            if nil ~= string.find(message, '#INTERRUPTED_SPELL#', 1, true) then
                message = string.gsub(message, '#INTERRUPTED_SPELL#', interruptedSpell);
            end
            if nil ~= string.find(message, '#INTERRUPTING_USER#', 1, true) then
                message = string.gsub(message, '#INTERRUPTING_USER#', interruptingUser);
            end

            -- send the mesage:
            SendChatMessage(message, "EMOTE");

        -----------------------------------------------------------------------
        -- other channel than 'Emote':
        -----------------------------------------------------------------------
        else
            local numberOfGroupMembers = GetNumGroupMembers();
            local inParty = numberOfGroupMembers > 0 and numberOfGroupMembers <= 5;
            local inRaid = numberOfGroupMembers > 5;

            --
            -- determine the channel for the mode 'Auto':
            --
            local autoChannel = "Self";
            if inParty then
                autoChannel = "PARTY";
            elseif inRaid then
                local ssInInstance, ssinstanceType = IsInInstance();
                -- http://www.wowwiki.com/API_IsInInstance
                -- results: none, pvp, arena, party, raid
                if ssinstanceType == "pvp" then
                    autoChannel = "BATTLEGROUND";
                else
                    autoChannel = "RAID"
                end
            end

            --
            -- determine output string
            --
            local interruptMessage = ToxynasIInterruptedThatDB.message;
            interruptMessage = string.gsub(
                    ToxynasIInterruptedThatDB.message, 'MOB', interruptedUser);
            interruptMessage = string.gsub(
                    interruptMessage, 'SPELL', GetSpellLink(spellID));
            if interruptingUser ~= UnitName("player") then
                interruptMessage = string.gsub(
                        interruptMessage, 'PERSON', interruptingUser);
            else
                interruptMessage = string.gsub(
                        interruptMessage, 'PERSON', UnitName("player"));
            end

            --
            -- Ausgabe der Nachricht abhängig von der gewählten Ausgabeform
            --
            if ToxynasIInterruptedThatDB.interruptOutput == 'Self'
                    or autoChannel == 'Self' then
                print(interruptMessage);
            elseif ToxynasIInterruptedThatDB.interruptOutput == 'Say' then
                if autoChannel == "pvp" then
                    print(interruptMessage);
                else
                    SendChatMessage(interruptMessage, "SAY");
                end
            elseif ToxynasIInterruptedThatDB.interruptOutput == 'Auto' then
                if (not inParty) and (not inRaid) then
                    print(interruptMessage);
                else
                    SendChatMessage(interruptMessage, autoChannel);
                end
            end
        end
    end
end

-------------------------------------------------------------------------------
-- Registers the command /tiit and handels user input starting with /tiit.   --
-------------------------------------------------------------------------------
SLASH_TIIT1 = "/tiit"
SlashCmdList["TIIT"] =
function(msg)
    local a1 = gsub(msg, "%s*([^%s]+).*", "%1");       -- first parameter
    local a2 = gsub(msg, "%s*([^%s]+)%s*(.*)", "%2");  -- second parameter
    a1 = string.lower(a1);  -- to lower characters 'eMotE' -> 'emote'

    if a1 == "" or a1 == "help" then
        TIIT_help();
    elseif a1 == "info" then
        TIIT_show_info();
    elseif a1 == "toggle" then
        TIIT_toggle_addon_active();
    elseif a1 == "say" then
        ToxynasIInterruptedThatDB.interruptOutput = 'Say';
        TIIT_announce_output_channel();
    elseif a1 == "self" then
        ToxynasIInterruptedThatDB.interruptOutput = 'Self';
        TIIT_announce_output_channel();
    elseif a1 == "auto" then
        ToxynasIInterruptedThatDB.interruptOutput = 'Auto';
        TIIT_announce_output_channel();
    elseif a1 == "emote" then
        ToxynasIInterruptedThatDB.interruptOutput = 'Emote';
        TIIT_announce_output_channel();
    elseif (a1 == "allmembers") then
        TIIT_toggle_allmembers();
    elseif (a1 == "msg") then
        print(TIIT_color .. L["the message is"] .. ": |r" ..
                ToxynasIInterruptedThatDB.message);
    elseif a1 == 'setmsg' then
        TIIT_set_message(a2);
    else
        print(TIIT_color .. TIIT_name .. " " .. L['parameter'] .. " |r" .. a1 ..
                " " .. TIIT_color .. L['is unknown'] .. ".|r");
    end
end

-------------------------------------------------------------------------------
-- function TIIT_help() shows the addon's options to the user.               --
-------------------------------------------------------------------------------
function TIIT_help()
    c_print(TIIT_name .. ' ' .. L['Version'] .. ' ' .. TIIT_version .. ' ' ..
            L['from'] .. ' ' .. TIIT_date ..'|r');
    c_print(L["To switch the interrupt message to a channel "
            .. "use /tiit with say, self, auto, or emote. Other commands are:"]);
    c_print(L["/tiit or /tiit help - show this message"]);
    c_print(L["/tiit info - show detailed informations"]);
    c_print(L["/tiit toggle - switches"] .. " " .. TIIT_name .. " " ..
            L["on or off"]);
    c_print(L["/tiit allmembers - switch interrupt reports from all "
            .. "members on or off."]);
    c_print(L["/tiit msg - show the message (not used for the "
            .. "'Emote'-mode)"]);
    c_print(L["/tiit setmsg - change the message. Use 'MOB' for the "
            .. "enemy's name, 'SPELL' for the enemy's spell you have "
            .. "interrupted, and 'PERSON' for the interrupting Person."]);
end

-------------------------------------------------------------------------------
-- The function TIIT_show_info() show's some state of the addon to the user. --
-------------------------------------------------------------------------------
function TIIT_show_info()
    c_print(TIIT_name .. " " .. L["Informations"] .. ":");
    if ToxynasIInterruptedThatDB.isActive ~= 1 then
        c_print(TIIT_name .. " is not active.");
    else
        TIIT_announce_output_channel();
        local TIIT_allmembers = TIIT_color
                .. L["Announce interrupts from all party members"] .. ": |r";
        if ToxynasIInterruptedThatDB.announceInterruptsFromAll == 1 then
            print(TIIT_allmembers .. " " .. L["On"]);
        else
            print(TIIT_allmembers .. " " .. L["Off"]);
        end
        print(TIIT_color .. L['message for modes other than Emote is'] .. ':|r'
                .. ToxynasIInterruptedThatDB.message);
    end
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_addon_active() toggles between activated and     --
-- inactivated state of the addon.                                           --
-------------------------------------------------------------------------------
function TIIT_toggle_addon_active()
    if ToxynasIInterruptedThatDB.isActive == 1 then
        ToxynasIInterruptedThatDB.isActive = 0;
        c_print(TIIT_name .. " " .. L["is now switched off."]);
    else
        ToxynasIInterruptedThatDB.isActive = 1;
        c_print(TIIT_name .. " " .. L["is now switched on."]);
    end
end

-------------------------------------------------------------------------------
-- The function TIIT_announce_output_channel() tells the output channel to   --
-- the user.                                                                 --
-------------------------------------------------------------------------------
function TIIT_announce_output_channel()
    print(TIIT_color .. L["Output channel is"] .. ": |r",
            ToxynasIInterruptedThatDB.interruptOutput);
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_allmembers() toggles between telling about       --
-- interrupts from other players and not telling so.                         --
-------------------------------------------------------------------------------
function TIIT_toggle_allmembers()
    if ToxynasIInterruptedThatDB.announceInterruptsFromAll == 1 then
        ToxynasIInterruptedThatDB.announceInterruptsFromAll = 0
        c_print(TIIT_name .. " - " ..
                L["Show interrupts from all members is switched off."]);
    else
        ToxynasIInterruptedThatDB.announceInterruptsFromAll = 1
        c_print(TIIT_name .. " - " ..
                L["Show interrupts from all members is switched on."]);
    end
end

-------------------------------------------------------------------------------
-- function TIIT_set_message() validates the user input and sets the new     --
-- message, if it is validated.                                              --
-------------------------------------------------------------------------------
function TIIT_set_message(message)
    local has_mob    = nil ~= string.find(message, "MOB",    1, true);
    local has_spell  = nil ~= string.find(message, "SPELL",  1, true);
    local has_person = nil ~= string.find(message, "PERSON", 1, true);
    if has_mob and has_spell and has_person then
        print(TIIT_color .. message .. " " .. L["is valid"] .. " |r");
        ToxynasIInterruptedThatDB.message = message;
        print(TIIT_color .. L["message is set to"] .. ": |r"
                .. ToxynasIInterruptedThatDB.message);
    else
        c_print(message .. " " .. L["is NOT valid (use MOB for the enemy, "
                .. "SPELL for the spell and PERSON for the interrupter "
                .. "as placeholders)"]);
        print(TIIT_color .. L["message stays"] .. ": |r"
                .. ToxynasIInterruptedThatDB.message);
    end
end
